@@ -11,19 +11,24 @@ import queue
 import string
 from xtea import MODE_CBC, new
 from colr import color
+import sys
 
 JOINTIMEOUT = 3
 RECEIVEMQSITZE = 10
 SERIALLEN = 16
 CRYPTOKEYLEN = 32
-ACKTIMEOUT = 3
+ACKTIMEOUT = 0.5
 BUFFERSIZE = 2048
 SOCKETRETRIES = 3
 SOCKETWAITTIME = 1
 KEEPALIVETIMEOUT = 25
 RECONNECTTIMEOUT = 5
 TIDBLOCKTIME = 90
-MAXIMUMSENDTIME = 3
+MAXIMUMSENDTIME = 1.5
+MAXIMUMTRANSACTIONTIME = 3
+MAXMESSAGEAGE = 5
+RECEIVEMSGQUEUSIZE = 2
+LOGDATABLOCKTIME = 120
 
 class DcsConnection:
     __serial__ = None
@@ -47,6 +52,7 @@ class DcsConnection:
     __keepConnected__ = False
     __device__ = None
     __logData__ = None
+    __logDataTimestamp__ = None
     
     def __init__(self, serial, cryptoKey, server, port):
         if serial != None and len(serial) == SERIALLEN:
@@ -179,18 +185,17 @@ class DcsConnection:
         logging.debug(str(id(self)) + " Receiver running...")
         while self.__running__ == True:
             wait = False
-            self.__socketLock__.acquire()
-            if(self.__socket__ != None and not self.__socket__._closed):
-                try:
-                    rec = self.__socket__.recv(BUFFERSIZE)
-                    self.__receiveBuffer__ += rec
-                except BlockingIOError as e:
-                    if e.errno != 10035 and e.errno != 11:
-                        logging.error(str(id(self)) + " BlockingIOError: " + str(e))
-                except Exception as e:
-                    logging.error(str(id(self)) + " Unknown error receiving data from socket. " + str(e))
-                    wait = True
-            self.__socketLock__.release()
+            with self.__socketLock__:
+                if(self.__socket__ != None and not self.__socket__._closed):
+                    try:
+                        rec = self.__socket__.recv(BUFFERSIZE)
+                        self.__receiveBuffer__ += rec
+                    except BlockingIOError as e:
+                        if e.errno != 10035 and e.errno != 11:
+                            logging.error(str(id(self)) + " BlockingIOError: " + str(e))
+                    except Exception as e:
+                        logging.error(str(id(self)) + " Unknown error receiving data from socket. " + str(e))
+                        wait = True     # set wait but don't wait here because mutex has to be released first
 
             if(wait == True):
                 wait = False
@@ -215,36 +220,48 @@ class DcsConnection:
                         "payload": rec_payload,
                         "receivetime": datetime.now()
                     }
-                    self.__receivedMessageQueue__.put(message)
+                    try:
+                        self.__receivedMessageQueue__.put_nowait(message)
+                    except Exception as e:
+                        logging.warning(str(id(self)) + " cannot put message in __receivedMessageQueue__: " + str(e))
                     self.__lastMessageTime__ = datetime.now()
 
-                    if self.__receivedMessageQueue__.qsize() >= 2:      #todo LIMIT als konstante
+                    if self.__receivedMessageQueue__.qsize() >= RECEIVEMSGQUEUSIZE:
                         logging.warning(str(id(self)) + " receivedMessageQueue size exceeded. Size: " + str(self.__receivedMessageQueue__.qsize()))
             else:
                 logging.debug(str(id(self)) + " nothing to receive.")
-                time.sleep(0.1)
+                time.sleep(0.001)
  
     #handling massages
     def __worker__(self):
         logging.debug(str(id(self)) + " Worker running...")
-        while self.__running__ == True:        
+        cleanupconter = 0
+        while self.__running__ == True:
+            if cleanupconter > 50:
+                try:
+                    self.__cleanSentrequests__()
+                    cleanupconter = 0
+                except Exception as e:
+                    logging.debug(str(id(self)) + " error cleaning mq: " + str(e))
+            cleanupconter += 1
+
             try:
                 message = self.__receivedMessageQueue__.get_nowait()
             except Exception:
                 message = None
-                time.sleep(0.1)
+                time.sleep(0.001)
 
             if(message):
                 logging.debug(str(id(self)) + " Handling Message: " + str(message))
                 try:
                     difference = datetime.now() - message["receivetime"]
-                    if(difference.total_seconds() < 5):     #Todo Konstante definieren
+                    if(difference.total_seconds() < MAXMESSAGEAGE or message["type"] == 9):
                         self.__handlemessage__(message["type"], message["payload"], message["tid"])
                         processingtime = datetime.now() - message["receivetime"]
-                        if(processingtime.total_seconds() > 5):     #Todo Konstante definieren
-                            logging.warning(str(id(self)) + " processing time too log: " + str(difference.total_seconds()))
+                        if(processingtime.total_seconds() > MAXMESSAGEAGE):
+                            logging.warning(str(id(self)) + " processing time too long: " + str(processingtime.total_seconds()) + ". MsgType: " + str(message["type"]) + ", TID: " + str(message["tid"]))
                     else:
-                        logging.warning(str(id(self)) + " message to old. Rejecting message. Age: " + str(difference.total_seconds()))
+                        logging.warning(str(id(self)) + " message to old. Rejecting message. Age: " + str(difference.total_seconds()) + ". MsgType: " + str(message["type"]) + ", TID: " + str(message["tid"]))
                 except Exception as e:
                     logging.error(str(id(self)) + " Error handling message. " + str(e))
 
@@ -287,14 +304,16 @@ class DcsConnection:
 
         if(msgtype == 6):
             logging.info(str(id(self)) + " TID: " + str(tid) + "; Received Calc Log Data (for MSG-Type 2).")
-            # if payload != None:
-            #     decryptedpayload = self.__decryptPayload__(payload)
+            if payload != None:
+                time = self.__decryptPayload__(payload)
+            else:
+                time = None
             nack=True
             try:
-                self.loadlogdata()
+                self.loadlogdata(time)
                 nack=False
             except Exception as e:
-                logging.warning(str(id(self)) + " TID: " + str(tid) + "; Error getting logData")
+                logging.warning(str(id(self)) + " TID: " + str(tid) + "; Error getting logData. " +  str(e))
             try:
                 self.__send__(None, 9, tid, isresponce=True, nack=nack)     #send ACK/NACK
             except Exception as e:
@@ -335,6 +354,7 @@ class DcsConnection:
         if(self.__workerThread__ != None and not self.__workerThread__.is_alive()):
             logging.warning(str(id(self)) + " Worker thread death. Restarting...")
             try:
+                self.__workerThread__.join(0.1)
                 self.__workerThread__ = self.__createWorkerThread__()
                 self.__workerThread__.start()
             except Exception as e:
@@ -359,20 +379,21 @@ class DcsConnection:
         logging.debug(str(id(self)) + " MSQ-Size: " + str(self.__receivedMessageQueue__.qsize()))
 
     def __cleanSentrequests__(self):
-        cleanList = []
-        for request in self.__sentRequests__:
-            try:
-                if self.__sentRequests__[request] != None and self.__sentRequests__[request]["timestamp"] < datetime.now() - timedelta(seconds=TIDBLOCKTIME):
-                    cleanList.append(request)
-            except Exception as e:
-                logging.debug(str(id(self)) + " request " + str(request) + " element does not exist anymore. " + str(e))
+        with self.__messageLock__:
+            cleanList = []
+            for request in self.__sentRequests__:
+                try:
+                    if self.__sentRequests__[request] != None and self.__sentRequests__[request]["timestamp"] < datetime.now() - timedelta(seconds=TIDBLOCKTIME):
+                        cleanList.append(request)
+                except Exception as e:
+                    logging.debug(str(id(self)) + " request " + str(request) + " element does not exist anymore. " + str(e))
 
-        for element in cleanList:
-            try:
-                logging.debug(str(id(self)) + " Cleaning up request " + str(element) + ".")
-                del self.__sentRequests__[element]
-            except Exception as e:
-                logging.debug(str(id(self)) + " Cannot clean request " + str(element) + ". " + str(e))
+            for element in cleanList:
+                try:
+                    logging.debug(str(id(self)) + " Cleaning up request " + str(element) + ".")
+                    del self.__sentRequests__[element]
+                except Exception as e:
+                    logging.debug(str(id(self)) + " Cannot clean request " + str(element) + ". " + str(e))
 
     def __dcsConnect__(self):
         if not self.__connected__:
@@ -457,18 +478,17 @@ class DcsConnection:
         self.__socketLock__.release()
 
     def __ackrequest__(self, tid):
-        self.__messageLock__.acquire()
-        try:
-            self.__sentRequests__[tid]["state"] = "ACK"
-            self.__sentRequests__[tid]["timestamp"] = datetime.now()
-        except Exception as e:
-            logging.warning(str(id(self)) + " TID: " + str(tid) + " Request not found. ACK not set. " + str(e))
+        with self.__messageLock__:
+            try:
+                self.__sentRequests__[tid]["state"] = "ACK"
+                self.__sentRequests__[tid]["timestamp"] = datetime.now()
+            except Exception as e:
+                logging.warning(str(id(self)) + " TID: " + str(tid) + " Request not found. ACK not set. " + str(e))
 
-        try:
-            self.__sentRequests__[tid]["event"].set()
-        except Exception as e:
-            logging.warning(str(id(self)) + " TID: " + str(tid) + " Request not found. Could not wakeup. " + str(e))
-        self.__messageLock__.release()
+            try:
+                self.__sentRequests__[tid]["event"].set()       #wakeup request
+            except Exception as e:
+                logging.warning(str(id(self)) + " TID: " + str(tid) + " Request not found. Could not wakeup. " + str(e))
 
     def __send__(self, payload, msgtype, tid, isresponce=False, nack=False):
         startsendtime = datetime.now()
@@ -481,18 +501,19 @@ class DcsConnection:
         except Exception as e:
             return False
         if package != None:
-            if self.__registermessage__(tid, msgtype):
-                try:
-                    self.__socketLock__.acquire()
-                    self.__socket__.send(package)
-                    # self.__socket__.sendall(package)
-                    self.__lastMessageSendTime__ = datetime.now()
-                    logging.debug(str(id(self)) + " TID: " + str(tid) + "; Type: " + str(msgtype) + " transmitted.")
-                    self.__socketLock__.release()
-                except Exception as e:
-                    logging.error(str(id(self)) + " cannot send. " + str(e))
-                    self.__socketLock__.release()
-                    return False
+            if self.__registermessage__(tid, msgtype, isresponce):
+                with self.__socketLock__:
+                    try:
+                        self.__socket__.send(package)
+                        self.__lastMessageSendTime__ = datetime.now()
+                        logging.debug(str(id(self)) + " TID: " + str(tid) + "; Type: " + str(msgtype) + " transmitted.")
+                    except Exception as e:
+                        logging.error(str(id(self)) + " cannot send. " + str(e))
+                        return False
+
+                difference = datetime.now() - startsendtime
+                if(difference.total_seconds() > MAXIMUMSENDTIME):
+                    logging.warning(str(id(self)) + " TID: " + str(tid) + "; Type: " + str(msgtype) + " sending time to hight! " + str(difference.total_seconds()))
                 
                 if(isresponce == False):
                     if(not self.__waitForAck__(tid)): 
@@ -500,10 +521,10 @@ class DcsConnection:
                         return False
 
                 difference = datetime.now() - startsendtime
-                if(difference.total_seconds() > MAXIMUMSENDTIME):
-                    logging.warning(str(id(self)) + " TID: " + str(tid) + "; Type: " + str(msgtype) + " sending time to hight! " + str(difference.total_seconds()))
+                if(difference.total_seconds() > MAXIMUMTRANSACTIONTIME):
+                    logging.warning(str(id(self)) + " TID: " + str(tid) + "; Type: " + str(msgtype) + " transaction time to hight! " + str(difference.total_seconds()))
 
-                logging.debug(str(id(self)) + " TID: " + str(tid) + "; Type: " + str(msgtype) + " successfull send.")
+                logging.info(str(id(self)) + " TID: " + str(tid) + "; Type: " + str(msgtype) + " successfull send. Transaction-time: " + str(difference.total_seconds()))
                 return True
             else:
                 return False
@@ -511,11 +532,10 @@ class DcsConnection:
             logging.error(str(id(self)) + " no package to send")
             return False
 
-    def __registermessage__(self, tid, msgtype):
-        if msgtype == 9:
+    def __registermessage__(self, tid, msgtype, isresponce=False):
+        if msgtype == 9 or isresponce==True:
             return True;
         with self.__messageLock__:
-            self.__cleanSentrequests__()
             try:
                 self.__sentRequests__[tid]
                 if self.__sentRequests__[tid] != None:
@@ -535,7 +555,7 @@ class DcsConnection:
     def __waitForAck__(self, tid):
         with self.__messageLock__:
             try:
-                if(self.__sentRequests__[tid]["state"] != "ACK"):
+                if(self.__sentRequests__[tid]["state"] != "ACK"):   #message has already been acked in the while
                     self.__sentRequests__[tid]["state"] = "SEND"
                     self.__sentRequests__[tid]["timestamp"] = datetime.now()
                     event = self.__sentRequests__[tid]["event"]
@@ -556,7 +576,7 @@ class DcsConnection:
                 del self.__sentRequests__[tid]
             except Exception as e:
                 logging.warning(str(id(self)) + " TID: " + str(tid) + " MSG not found. " + str(e))
-        return response
+            return response
 
     def __wakeUpWaitingMessages__(self):
         self.__socketLock__.acquire()
@@ -593,7 +613,6 @@ class DcsConnection:
         try:
             rand = self.__get_random_string__(8)
             iv = rand.encode('utf-8')
-            # iv = "50976281".encode('utf-8')     #Todo: woher kommt die Zahl?
         except Exception as e:
             logging.error("TID: " + str(tid) + " Error generating IV: " + str(e))
             raise(e)
@@ -632,9 +651,13 @@ class DcsConnection:
         return result_str 
 
     def __decryptPayload__(self, payload):
-        xtea = new(self.__cryptoKey__, mode=MODE_CBC,IV=payload[32:40])
-        payload_decrypted = xtea.decrypt(payload)[8:]
+        iv=payload[(len(payload)-8):]
+        xtea = new(self.__cryptoKey__, mode=MODE_CBC,IV=iv)
+        payload_decrypted = xtea.decrypt(payload)[8:(len(payload)-2)]
         payload_string = payload_decrypted.decode()
+        return payload_string
+
+    def __getsetupelements__(self, payload_string):
         message_string = payload_string[payload_string.find("?")+1:]
         setup_string = message_string[0:message_string.find("/")]
         logging.debug("["+str(self.__serial__ )+"@"+str(self.__serverAddress__)+":"+str(self.__serverPort__)+"]; Setup changes: " + str(setup_string))
@@ -643,7 +666,10 @@ class DcsConnection:
 
     def senddata(self, tid=None):
         if(self.__connected__):
-            data = self.__getCurrentData__()
+            try:
+                data = self.__getCurrentData__()
+            except Exception as e:
+                logging.warning(str(id(self)) + " TID: " + str(tid) + ", cannot get CurrentData: " + str(e))
             if(tid == None):
                 isresponce=True
                 tid = self.__getTid__()
@@ -691,16 +717,15 @@ class DcsConnection:
             raise Exception(errormsg)
 
     def __getTid__(self):
-        self.__messageLock__.acquire()
-        found = None
-        while found == None:
-            tid = random.randint(1, 65535)
-            try:
-                self.__sentRequests__[tid]
-            except:
-                found = tid
-                self.__sentRequests__[tid] = None
-        self.__messageLock__.release()
+        with self.__messageLock__:
+            found = None
+            while found == None:
+                tid = random.randint(1, 65535)
+                try:
+                    self.__sentRequests__[tid]
+                except:
+                    found = tid
+                    self.__sentRequests__[tid] = None
         return found
 
     def sendsetup(self, forced = None, tid=None):
@@ -740,13 +765,27 @@ class DcsConnection:
         else:
             return self.__device__.getdata()
     
-    def loadlogdata(self):
+    def loadlogdata(self, time = None):
+
+        if self.__logDataTimestamp__ != None:
+            difference = datetime.now() - self.__logDataTimestamp__
+            if(difference.total_seconds() < LOGDATABLOCKTIME): 
+                logging.debug(str(id(self)) + " cannot get logdata. Minimum waiting time not reached. " + str(difference.total_seconds()))
+                raise Exception("Logdata not ready.")
+
+        t = datetime.now()
+        if time == None:
+            time = t.strftime("%Y%m%d%H%M%S")
+        else:
+            time = str(time)
+            
         if(self.__device__ == None):
             logging.warning(str(id(self)) + " device not found. getting dummy logData")
-            t = datetime.now()
-            self.__logData__ = {"i_power": t.second * 100, "time": str(t)}
+            self.__logData__ = {"i_power": t.second * 100, "time": time}
         else:
-            self.__logData__ = self.__device__.getlogdata()
+            self.__logData__ = self.__device__.getlogdata(time)
+        
+        self.__logDataTimestamp__ = t
 
     def clearlogdata(self):
         if(self.__device__ == None):
@@ -759,7 +798,8 @@ class DcsConnection:
 
     def __changeSetup__(self, payload):
         try:
-            setupChanges = self.__decryptPayload__(payload)
+            setupstring = self.__decryptPayload__(payload)
+            setupChanges = self.__getsetupelements__(setupstring)
             logging.debug(str(id(self)) + " Changing setup: " + str(setupChanges))
         except Exception as e:
             error = str(id(self)) + " Error decoding setup changes: " + str(e)
@@ -791,6 +831,31 @@ if __name__ == "__main__":
     cryptoKeyInvalid = "ABABABABABABAB00ABABABABABABAB00"
     server = "my-pv.live"
 
+    if True:
+        connection = DcsConnection(serial, cryptoKey, server, 50333)
+        connection2 = DcsConnection(serial2, cryptoKey, server, 50333)
+        connection.connect()
+        connection2.connect()
+        hash1 = connection.getsockethash()
+        hash2 = connection2.getsockethash()
+        try:
+            while True:
+                print(color('[DCS-Connection] Communication active. Press CTRL+C to stop', fore='blue', style='bright'))
+                # connection.watchdog()
+                # connection2.watchdog()
+                if(hash1 != connection.getsockethash()):
+                    print(color('[DCS-Connection] Socket of serial ' + str(serial) + ' has changed.', fore='red', style='bright'))
+                    hash1 = connection.getsockethash()
+                if(hash2 != connection2.getsockethash()):
+                    print(color('[DCS-Connection] Socket of serial ' + str(serial2) + ' has changed.', fore='red', style='bright'))
+                    hash2 = connection2.getsockethash()
+
+                time.sleep(10)
+        except KeyboardInterrupt as e:
+            print("[DCS-Connection] Stopping Test...")
+            connection.disconnect()
+            connection2.disconnect()
+
     #AUTO-Tests
     #Constructor Tests
     try:
