@@ -15,11 +15,11 @@ sys.path.append(os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)))
 
 from mypvdevices.DcsConnection import DcsConnection
 
-BUILDNR = "1.2102.5"
+BUILDNR = "1.2102.13"
 
 JOINTIMEOUT = 5
 SERIALLEN = 16
-DEVICESLEEPTIME = 5
+DEVICESLEEPTIME = 3
 DEFAULTMODBUSFREQUENCY = 3
 
 class ModbusDevice:
@@ -91,7 +91,7 @@ class ModbusDevice:
         self.__monitorthread__.start()
         self.__modbusThread__ = threading.Thread(target=self.__runmodbus__, name="Modbus " + str(self.__serial__))
         self.__modbusThread__.start()
-        time.sleep(1)
+        time.sleep(0.1)
         for connection in self.__connections__:
             connection.connect()
         logging.debug("[ModbusDevice] ID " + str(self.__serial__) + " started.")
@@ -187,14 +187,17 @@ class ModbusDevice:
             try:
                 self.__supervise__()
             except Exception as e:
-                logging.warning("[ModbusDevice] ID " + str(self.__serial__) + " supervision failed failed. " + str(e))
+                logging.warning("[ModbusDevice] ID " + str(self.__serial__) + " supervision failed. " + str(e))
 
-            if self.__modBusExecuted__ != None:
-                difference = datetime.now() - self.__modBusExecuted__
-                if(difference.total_seconds() > self.__modbusFrequency__ * 3):
-                    if self.__datasets__ != None:
-                        for dataset in self.__datasets__:
-                            self.__data__[dataset] = None
+            try:
+                if self.__modBusExecuted__ != None:
+                    difference = datetime.now() - self.__modBusExecuted__
+                    if(difference.total_seconds() > self.__modbusFrequency__ * 3):
+                        if self.__datasets__ != None:
+                            for dataset in self.__datasets__:
+                                self.__data__[dataset] = None
+            except Exception as e:
+                logging.warning("[ModbusDevice] ID " + str(self.__serial__) + " cleanup failed. " + str(e))
 
             time.sleep(DEVICESLEEPTIME)
 
@@ -273,7 +276,11 @@ class ModbusDevice:
     def __excecutemodbus__(self):
         if self.__modbusFrequency__ > 0:
             logging.debug("[ModbusDevice] ID " + str(self.__serial__) + " Modbus running...")
-            self.__readallregisters__()
+            try:
+                self.__readallregisters__()
+            except Exception as e:
+                logging.error("[ModbusDevice] " + self.getidentifier() + ". Unknown modbus register read error: " + str(e))
+                self.__registers__ = None
             if self.__registers__ != None:
                 if len(self.__registers__) > 0:
                     try:
@@ -318,7 +325,7 @@ class ModbusDevice:
         if errors == 0:
             return True
         else:
-            return False
+            raise Exception("Errors with " + str(errors) + " elements")
 
     def sendsetup(self):
         success = True
@@ -568,9 +575,10 @@ class ModbusDevice:
                 }            
         return data
 
-    def getlogdata(self):
+    def getlogdata(self, time = None):
         logdata = {
             # "i_unknown": 123,
+            "time": time,
             "i_power": self.getintlogvalue("power"),
             "i_boostpower": 750,
             "i_meterfeed": None,
